/*
 * Server.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: real_vg, TCSDEVELOPER
 */

using System;
using System.Net;
using NAnt.Core;
using NAnt.Core.Types;

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// Provides an abstract base class for all the classes representing
    /// a server to be used when running tests.
    /// </summary>
    ///
    /// <remarks>
    /// It stores some attributes passed from the <see cref="ServerElement"/> class.
    /// The appropriate properties are provided for accessing the attributes.
    /// It also has properties for accessing the nested <see cref="SourceFiles"/> and
    /// <see cref="Credentials"/> elements of the "server" XML element.
    /// The other important thing is a property for specifying the options of the server.
    /// The class also provides the abstract methods of starting/stopping/initializing/cleaning-up data,
    /// which should be implemented by derived classes.
    /// </remarks>
    ///
    /// <threadsafety static="false" instance="false"/>
    public abstract class Server
    {
        /// <summary>
        /// A string constant that represents the name (key) of the option, used by derived classes,
        /// for getting the timeout, which is a time in milliseconds, the <see cref="StartUp"/> method
        /// should wait for the server to start.
        /// </summary>
        ///
        /// <remarks>
        /// If the specified time passes and the server is not running, the <see cref="StartUp"/>
        /// method should throw an exception.
        /// The behaviour described here is only a suggestion, because derived classes
        /// can treat options in any way they like to do.
        /// </remarks>
        public static readonly string TIMEOUT = "timeout";

        /// <summary>
        /// A string constant that represents the name (key) of the option, used by derived classes,
        /// for getting the port number, used to connect to the server.
        /// </summary>
        ///
        /// <remarks>
        /// Note that derived classes can treat options in any way they like to do.
        /// </remarks>
        public static readonly string PORT = "port";

        /// <summary>
        /// A string constant that represents the name (key) of the option, used by derived classes,
        /// for getting the address of the machine running the server.
        /// </summary>
        ///
        /// <remarks>
        /// Note that derived classes can treat options in any way they like to do.
        /// </remarks>
        public static readonly string ADDRESS = "address";

        /// <summary>
        /// This variable represents the "startserver" attribute.
        /// It specifies if the server should be started by the component when running tests.
        /// This attribute is not required, the default value is true.
        /// Can be accessed via the StartServer property.
        /// </summary>
        private bool startServer = true;

        /// <summary>
        /// This variable represents the "initserver" attribute.
        /// It specifies if the server should be initialized with data by the component when running tests.
        /// This attribute is not required, the default value is true.
        /// Can be accessed via the InitServer property.
        /// </summary>
        private bool initServer = true;

        /// <summary>
        /// <para>
        /// This variable represents the collection of nested "option" XML elements, which
        /// represents the options of the server.
        /// The options can be any key-value pairs, where keys and values are strings.
        /// Initially is null. Can be accessed via the "Options" property.
        /// </para>
        /// <para>
        /// The derived classes may interpret the options in a free way.
        /// The string constants representing the names of the most common
        /// options are provided by the class.
        /// They are : TIMEOUT, PORT, and ADDRESS.
        /// The derived classes should provide a similar string constants
        /// representing the names of options they need.
        /// </para>
        /// </summary>
        private OptionCollection options;

        /// <summary>
        /// This variable represents the nested "credentials" XML element, which
        /// represents the credentials used to authenticate to the server.
        /// This includes user name, password, and domain.
        /// This nested element can possibly be omitted,
        /// it depends on the particular server.
        /// Initially is null. Can be accessed via the "Credentials" property.
        /// </summary>
        private Credential credentials;

        /// <summary>
        /// This variable represents the nested "sourcefiles" XML element, which
        /// represents the files used when initializing the server with data.
        /// This can be an application to deploy for an application server; or
        /// some SQL scripts, DB tables, etc, for a data server; or some other files
        /// the particular server may need to be initialized.
        /// This nested element can possibly be omitted if the
        /// class derived from Server permits it.
        /// Initially is null. Can be accessed via the "SourceFiles" property.
        /// </summary>
        private FileSet sourceFiles;

        /// <summary>
        /// This variable represents the NAnt project this Server belongs to.
        /// Initially is null. Can be accessed via the "Project" property.
        /// </summary>
        private Project project;

        /// <summary>
        /// Gets or sets a boolean value that indicates whether
        /// the server should be started by the component when running tests.
        /// </summary>
        ///
        /// <remarks>
        /// Implementation should ignore this value because this value is only
        /// used by <see cref="ServerAppTestTask"/> to determine whether
        /// <see cref="StartUp"/> should be called.
        /// </remarks>
        ///
        /// <value>
        /// true if the server should be started by the component when running tests;
        /// otherwise, false. This attribute is not required and the default value is true.
        /// </value>
        public bool StartServer
        {
            get
            {
                return startServer;
            }
            set
            {
                startServer = value;
            }
        }

        /// <summary>
        /// Gets or sets a boolean value that indicates whether
        /// the server should be initialized with data by the component when running tests.
        /// </summary>
        ///
        /// <value>
        /// true if the server should be initialized with data by the component when running tests;
        /// otherwise, false. This attribute is not required and the default value is true.
        /// </value>
        public bool InitServer
        {
            get
            {
                return initServer;
            }
            set
            {
                initServer = value;
            }
        }

        /// <summary>
        /// Gets or sets the collection of nested "option" XML elements, which
        /// represents the options of the server.
        /// </summary>
        ///
        /// <remarks>
        /// The string constants representing the names of the most common
        /// options are provided by the class.
        /// They are : TIMEOUT, PORT, and ADDRESS.
        /// The derived classes should provide a similar string constants
        /// representing the names of options they need.
        /// </remarks>
        ///
        /// <value>
        /// An OptionCollection which represents the options of the server.
        /// It can be any key-value pairs, where key and values are strings.
        /// It could also be null.
        /// </value>
        public OptionCollection Options
        {
            get
            {
                return options;
            }
            set
            {
                options = value;
            }
        }

        /// <summary>
        /// Gets or sets the "credentials" XML element, which
        /// represents the credentials used to authenticate to the server.
        /// </summary>
        ///
        /// <value>
        /// A Credential which represents the credentials used to authenticate to the server.
        /// It could be null.
        /// </value>
        public Credential Credentials
        {
            get
            {
                return credentials;
            }
            set
            {
                credentials = value;
            }
        }

        /// <summary>
        /// Gets or sets the "sourcefiles" XML element, which
        /// represents the files used when initializing the server with data.
        /// </summary>
        ///
        /// <remarks>
        /// This can be an application to deploy for an application server; or
        /// some SQL scripts, DB tables, etc, for a data server; or some other files
        /// the particular server may need to be initialized.
        /// </remarks>
        ///
        /// <value>
        /// A FileSet which represents the files used when initializing the server with data.
        /// It could be null.
        /// </value>
        public FileSet SourceFiles
        {
            get
            {
                return sourceFiles;
            }
            set
            {
                sourceFiles = value;
            }
        }

        /// <summary>
        /// Gets or sets the NAnt Project this Server belongs to.
        /// </summary>
        ///
        /// <value>
        /// The NAnt Project this Server belongs to.
        /// It could be null.
        /// </value>
        public Project Project
        {
            get
            {
                return project;
            }
            set
            {
                project = value;
            }
        }

        /// <summary>
        /// Gets the timeout value from the <see cref="Options"/> collection.
        /// </summary>
        ///
        /// <value>
        /// An integer representing the timeout in milliseconds that the StartUp() method
        /// should wait for the server to start.
        /// </value>
        ///
        /// <exception cref="ServerException">
        /// The value doesn't exist or is illegal or is a negative value.
        /// </exception>
        public int Timeout
        {
            get
            {
                // parse "timeout" option
                // must exists, can be parsed to integer, and non-negative
                return OptionHelper.GetInt(TIMEOUT, Options);
            }
        }

        /// <summary>
        /// Gets the port number used to connect to the server.
        /// </summary>
        ///
        /// <value>
        /// An integer representing the port number used to connect to the server.
        /// </value>
        ///
        /// <exception cref="ServerException">
        /// The value doesn't exist or is illegal or is not between
        /// <see cref="IPEndPoint.MinPort"/> and <see cref="IPEndPoint.MaxPort"/> inclusive.
        /// </exception>
        public int Port
        {
            get
            {
                // parse "port" option
                // must exists, can be parsed to integer, and between MinPort(0) & MaxPort
                int port = OptionHelper.GetInt(PORT, Options);
                if (port > IPEndPoint.MaxPort)
                {
                    throw new ServerException(string.Format(
                        "Port number ({0}) must be between {1} and {2}",
                        port, IPEndPoint.MinPort, IPEndPoint.MaxPort));
                }
                return port;
            }
        }

        /// <summary>
        /// Gets the address of the machine running the server.
        /// </summary>
        ///
        /// <value>
        /// The address of the machine running the server.
        /// </value>
        ///
        /// <exception cref="ServerException">
        /// The value doesn't exist.
        /// </exception>
        public string Address
        {
            get
            {
                // get the option value (must exists)
                return OptionHelper.GetString(ADDRESS, Options, false);
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Server"/> class.
        /// </summary>
        protected Server()
        {
            // do nothing
        }

        /// <summary>
        /// Starts the server.
        /// </summary>
        ///
        /// <remarks>
        /// Note that if <see cref="StartServer"/> is false,
        /// this method is never called by <see cref="ServerAppTestTask"/>.
        /// Subclasses must implement this method to perform the actual task of
        /// starting up the server.
        /// </remarks>
        ///
        /// <exception cref="ServerException">
        /// Any error happens.
        /// </exception>
        public abstract void StartUp();

        /// <summary>
        /// Stops the server.
        /// </summary>
        ///
        /// <remarks>
        /// Note that if <see cref="StartServer"/> is false,
        /// this method is never called by <see cref="ServerAppTestTask"/>.
        /// Subclasses must implement this method to perform the actual task of
        /// stopping the server.
        /// </remarks>
        ///
        /// <exception cref="ServerException">
        /// Any error happens.
        /// </exception>
        public abstract void ShutDown();

        /// <summary>
        /// Initializes the server with data.
        /// </summary>
        ///
        /// <remarks>
        /// Initialization will not occur if <see cref="InitServer"/> is false.
        /// Subclasses must implement this method to perform the actual task of
        /// initializing server with data.
        /// </remarks>
        ///
        /// <exception cref="ServerException">
        /// Any error happens.
        /// </exception>
        public abstract void InitData();

        /// <summary>
        /// Cleans up data initialized by <see cref="InitData"/>.
        /// </summary>
        ///
        /// <remarks>
        /// Cleans up will not occur if data was not initialized
        /// (i.e. <see cref="InitServer"/> is false).
        /// Subclasses must implement this method to perform the actual task of
        /// cleaning up the data.
        /// </remarks>
        ///
        /// <exception cref="ServerException">
        /// Any error happens.
        /// </exception>
        public abstract void CleanUpData();

        /// <summary>
        /// Gets and parses an option value for the specified key.
        /// </summary>
        ///
        /// <param name="key">
        /// The option key whose value to parse.
        /// </param>
        ///
        /// <returns>
        /// The parsed option value.
        /// </returns>
        ///
        /// <exception cref="ServerException">
        /// The option value doesn't exist or cannot be parsed to an integer value
        /// or is negative.
        /// </exception>
        private int ParseIntOption(string key)
        {
            // get the option value (required)
            Option opt = ((Options == null) ? null : Options[key]);
            if (opt == null)
            {
                throw new ServerException(string.Format(
                    "Missing required '{0}' option", key));
            }

            // parse it (cannot be negative)
            try
            {
                int intVal = int.Parse(opt.Value);
                if (intVal < 0)
                {
                    throw new ServerException(string.Format(
                        "'{0}' value ({1}) cannot be negative", key, intVal));
                }
                return intVal;
            }
            catch (ServerException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new ServerException(string.Format(
                    "Error parsing '{0}' option value '{1}' to integer", key, opt.Value), e);
            }
        }
    }
}
