/*
 * Demo.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System.IO;
using System.ServiceProcess;
using System.Xml;
using NAnt.Core;
using NUnit.Framework;

namespace TopCoder.Test.Framework.Tests
{
    /// <summary>
    /// A demonstration of component functionality.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// This demonstrates requires an IIS server.
    /// This will deploy the files in test_files\deploysource to the web server,
    /// start the server, run NUnitAsp test cases, then shut down and clean up the server.
    /// </para>
    /// <para>
    /// The NAnt build file test_files\ServerAppTest.build is used for this demonstration.
    /// The IIS web root path is specified in that file as $deployment_dir property.
    /// Please adjust it accordingly.
    /// </para>
    /// </remarks>
    [TestFixture]
    public class Demo : TestHelper
    {
        /// <summary>
        /// The deployment_dir as specified in test_files\ServerAppTest.build.
        /// Initialized in SetUp().
        /// </summary>
        private string deployment_dir;

        /// <summary>
        /// Makes sure that IIS is stopped and the webroot is deleted.
        /// This is used by both SetUp and TearDown.
        /// </summary>
        [SetUp, TearDown]
        public void SetUp()
        {
            ServiceHelper(ServiceControllerStatus.Stopped, new ServiceController("w3svc"));

            // read ServerAppTest.build and get the deployment_dir
            XmlDocument doc = new XmlDocument();
            doc.Load(SampleNAnt);
            XmlNode prop = doc.SelectSingleNode("/project/property[@name='deployment_dir']");
            deployment_dir = prop.Attributes["value"].Value;

            DeleteDir(deployment_dir);
        }

        /// <summary>
        /// Demonstrates component by executing a NAnt build file
        /// containing a ServerAppTestTask which prepares/cleans the servers and
        /// executes an NUnitAsp test against IIS.
        /// </summary>
        [Test]
        public void DemoWithNUnitAsp()
        {
            // allow test cases to be run by creating an empty file
            using (FileStream stream = File.Create(DemoNUnitAsp.RunByDemoFile))
            {
            }

            try
            {
                string result;

                // log is set to Info so we could assert console output that the NUnitAsp are run
                using (ConsoleCap c = new ConsoleCap())
                {
                    // this is similar to executing on the command line:
                    // nant -buildfile:ServerTestTask.build
                    Project project = new Project(SampleNAnt, Level.Info);
                    project.Execute();
                    result = c.Output;
                }

                // assert NAnt output:
                // 3 files are deployed/deleted and serverapptest is executed
                Assert.IsTrue(result.IndexOf("[copy] Copying 3 files to ") > 0,
                    "CopyTask not found: " + result);

                Assert.IsTrue(result.IndexOf("[serverapptest] Tests run: 3, Failures: 0, Not run: 0, Time: ") > 0,
                    "ServerAppTest task should test 3 pages: " + result);

                Assert.IsTrue(result.IndexOf("In DemoHello") > 0,
                    "DemoHello is not executed: " + result);

                Assert.IsTrue(result.IndexOf("In DemoPlain") > 0,
                    "DemoPlain is not executed: " + result);

                Assert.IsTrue(result.IndexOf("In DemoSub") > 0,
                    "DemoSub is not executed: " + result);

                Assert.IsTrue(result.IndexOf("[delete] Deleting 3 files and 2 directories.") > 0,
                    "DeleteTask not found: " + result);

                // assert that IIS is stopped
                Assert.AreEqual(ServiceControllerStatus.Stopped, new ServiceController("w3svc").Status,
                    "IIS should be stopped");

                // assert deployment_dir is deleted
                DirectoryInfo dir = new DirectoryInfo(deployment_dir);
                Assert.IsFalse(dir.Exists, "Deployment_dir is deleted");
            }
            finally
            {
                // clean up the file
                File.Delete(DemoNUnitAsp.RunByDemoFile);
            }
        }
    }
}
