/*
 * TestDatabaseServer.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using NAnt.Core.Types;
using NUnit.Framework;

namespace TopCoder.Test.Framework.Servers.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="DatabaseServer"/> class.
    /// </summary>
    ///
    /// <remarks>
    /// Since <see cref="DatabaseServer"/> is an abstract class,
    /// it is tested via the empty <see cref="MyDatabaseServer"/> class.
    /// </remarks>
    [TestFixture]
    public class TestDatabaseServer
    {
        /// <summary>
        /// The DatabaseServer instance to test. Initialized in SetUp to MyDatabaseServer instance.
        /// </summary>
        private DatabaseServer server;

        /// <summary>
        /// A sample Option collection. Initialized in SetUp.
        /// </summary>
        private OptionCollection options;

        /// <summary>
        /// Initializes the <see cref="DatabaseServer"/> instance to test
        /// via <see cref="MyDatabaseServer"/> and prepares a sample option collection.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            server = new MyDatabaseServer();

            // prepare a sample option collection
            options = new OptionCollection();

            // db_driver
            Option optDriver = new Option();
            optDriver.OptionName = DatabaseServer.DB_DRIVER;
            optDriver.Value = "postgresql";
            options.Add(optDriver);

            // db_connection_str
            Option optConn = new Option();
            optConn.OptionName = DatabaseServer.DB_CONNECTION_STR;
            optConn.Value = "host=localhost;user=tcs;password=tcs";
            options.Add(optConn);

            // sql_init_script
            Option optInit = new Option();
            optInit.OptionName = DatabaseServer.SQL_INIT_SCRIPT;
            optInit.Value = "init.sql";
            options.Add(optInit);

            // sql_cleanup_script
            Option optCleanup = new Option();
            optCleanup.OptionName = DatabaseServer.SQL_CLEANUP_SCRIPT;
            optCleanup.Value = "cleanup.sql";
            options.Add(optCleanup);
        }

        /// <summary>
        /// Tests static constants definitions.
        /// </summary>
        [Test]
        public void TestStatic()
        {
            Assert.AreEqual("db_driver", DatabaseServer.DB_DRIVER,
                "DB_DRIVER value is incorrect");
            Assert.AreEqual("db_connection_str", DatabaseServer.DB_CONNECTION_STR,
                "DB_CONNECTION_STR value is incorrect");
            Assert.AreEqual("sql_init_script", DatabaseServer.SQL_INIT_SCRIPT,
                "SQL_INIT_SCRIPT value is incorrect");
            Assert.AreEqual("sql_cleanup_script", DatabaseServer.SQL_CLEANUP_SCRIPT,
                "SQL_CLEANUP_SCRIPT value is incorrect");
        }

        /// <summary>
        /// Tests DatabaseDriver property. Make sure it retrieves OptionCollection correctly.
        /// </summary>
        [Test]
        public void TestDatabaseDriver()
        {
            // set the options first and assert
            server.Options = options;
            Assert.AreEqual("postgresql", server.DatabaseDriver,
                "DatabaseDriver value as in the options");
        }

        /// <summary>
        /// Tests DatabaseDriver property given DatabaseServer with null option collection.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestDatabaseDriverNullCollection()
        {
            string val = server.DatabaseDriver;
        }

        /// <summary>
        /// Tests DatabaseDriver property given option collection that contains no db_driver key.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestDatabaseDriverNotExists()
        {
            // remove db_driver from options
            options.Remove(options[DatabaseServer.DB_DRIVER]);
            server.Options = options;
            string val = server.DatabaseDriver;
        }

        /// <summary>
        /// Tests DatabaseConnectionString property. Make sure it retrieves OptionCollection correctly.
        /// </summary>
        [Test]
        public void TestDatabaseConnectionString()
        {
            // set the options first and assert
            server.Options = options;
            Assert.AreEqual("host=localhost;user=tcs;password=tcs", server.DatabaseConnectionString,
                "DatabaseConnectionString value as in the options");
        }

        /// <summary>
        /// Tests DatabaseConnectionString property given DatabaseServer with null option collection.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestDatabaseConnectionStringNullCollection()
        {
            string val = server.DatabaseConnectionString;
        }

        /// <summary>
        /// Tests DatabaseConnectionString property given option collection that contains
        /// no db_connection_str key. Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestDatabaseConnectionStringNotExists()
        {
            // remove db_connection_str from options
            options.Remove(options[DatabaseServer.DB_CONNECTION_STR]);
            server.Options = options;
            string val = server.DatabaseConnectionString;
        }

        /// <summary>
        /// Tests SqlInitScript property. Make sure it retrieves OptionCollection correctly.
        /// </summary>
        [Test]
        public void TestSqlInitScript()
        {
            // set the options first and assert
            server.Options = options;
            Assert.AreEqual("init.sql", server.SqlInitScript,
                "SqlInitScript value as in the options");
        }

        /// <summary>
        /// Tests SqlInitScript property given DatabaseServer with null option collection.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestSqlInitScriptNullCollection()
        {
            string val = server.SqlInitScript;
        }

        /// <summary>
        /// Tests SqlInitScript property given option collection that contains
        /// no sql_init_script key. Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestSqlInitScriptNotExists()
        {
            // remove sql_init_script from options
            options.Remove(options[DatabaseServer.SQL_INIT_SCRIPT]);
            server.Options = options;
            string val = server.SqlInitScript;
        }

        /// <summary>
        /// Tests SqlCleanUpScript property. Make sure it retrieves OptionCollection correctly.
        /// </summary>
        [Test]
        public void TestSqlCleanUpScript()
        {
            // set the options first and assert
            server.Options = options;
            Assert.AreEqual("cleanup.sql", server.SqlCleanUpScript,
                "SqlCleanUpScript value as in the options");
        }

        /// <summary>
        /// Tests SqlCleanUpScript property given DatabaseServer with null option collection.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestSqlCleanUpScriptNullCollection()
        {
            string val = server.SqlCleanUpScript;
        }

        /// <summary>
        /// Tests SqlCleanUpScript property given option collection that contains
        /// no sql_cleanup_script key. Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestSqlCleanUpScriptNotExists()
        {
            // remove sql_cleanup_script from options
            options.Remove(options[DatabaseServer.SQL_CLEANUP_SCRIPT]);
            server.Options = options;
            string val = server.SqlCleanUpScript;
        }
    }
}
