/*
 * TestIISServer.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System.IO;
using System.ServiceProcess;
using NAnt.Core;
using NAnt.Core.Types;
using NUnit.Framework;
using TopCoder.Test.Framework.Tests;

namespace TopCoder.Test.Framework.Servers.Web.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="IISServer"/> class.
    /// </summary>
    ///
    /// <remarks>
    /// This will attempt to deploy the files in test_files\deploysource
    /// to test_files\deploytarget.
    /// An IIS server must be installed on the localhost as well to test
    /// starting and stopping it.
    /// </remarks>
    [TestFixture]
    public class TestIISServer : TestHelper
    {
        /// <summary>
        /// The deployment source (relative from the test dll).
        /// </summary>
        private const string SourcePath = @"..\..\test_files\deploysource";

        /// <summary>
        /// The deployment target (relative from the test dll).
        /// </summary>
        private const string DeployPath = @"..\..\test_files\deploytarget";

        /// <summary>
        /// The IISServer instance to test. Initialized in SetUp.
        /// </summary>
        private IISServer server;

        /// <summary>
        /// A sample Option collection. Initialized in SetUp.
        /// </summary>
        private OptionCollection options;

        /// <summary>
        /// The IIS service. Initialized in SetUp.
        /// </summary>
        private ServiceController service;

        /// <summary>
        /// Initializes the <see cref="IISServer"/> instance to test
        /// and prepares a sample option collection.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // init IIS service controller first
            service = new ServiceController("w3svc");

            // delete everything in test_files\deploytarget and stop IIS
            TearDown();

            server = new IISServer();

            // create a dummy project using ..\..\test_files\ServerAppTest.build
            // the BaseDirectory will be the build file location: test_files
            server.Project = new Project(SampleNAnt, Level.Error);

            // prepare a sample option collection
            options = new OptionCollection();
            server.Options = options;

            // deployment_dir: deploytarget (relative from project basedir)
            Option optDeploy = new Option();
            optDeploy.OptionName = IISServer.DEPLOYMENT_DIR;
            optDeploy.Value = "deploytarget";
            options.Add(optDeploy);

            // sourcefiles: deploysource
            FileSet sourceFiles = new FileSet();
            sourceFiles.BaseDirectory = server.Project.GetFullPath("deploysource");
            sourceFiles.Includes.Add("*\\**");
            server.SourceFiles = sourceFiles;
        }

        /// <summary>
        /// Cleans up by deleting everything in test_files\deploytarget including the directory
        /// and stopping the IIS service.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            DeleteDir(DeployPath);
            ServiceHelper(ServiceControllerStatus.Stopped, service);
        }

        /// <summary>
        /// Tests static constants definitions and our initial test environment.
        /// </summary>
        [Test]
        public void TestStaticAndInitialEnvironment()
        {
            Assert.AreEqual("service_name", IISServer.SERVICE_NAME, "SERVICE_NAME value is incorrect");

            // iis is stopped
            service.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Stopped, service.Status, "IIS is stopped");

            // no deploytarget
            DirectoryInfo dir = new DirectoryInfo(DeployPath);
            Assert.IsFalse(dir.Exists, "Deploy target should not exist");
        }

        /// <summary>
        /// Tests StartUp. Assert that files are deployed and IIS service is started.
        /// </summary>
        [Test]
        public void TestStartUp()
        {
            server.StartUp();

            AssertFile(Path.Combine(SourcePath, "Notes.txt"), Path.Combine(DeployPath, "Notes.txt"));

            // assert deployed files and that iis is started
            AssertDeployedFiles();
            service.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Running, service.Status, "IIS should be started");
        }

        /// <summary>
        /// Tests StartUp given service that is in paused state.
        /// Assert that files are deployed and IIS service is started.
        /// </summary>
        [Test]
        public void TestStartUpPause()
        {
            // start then pause it
            ServiceHelper(ServiceControllerStatus.Running, service);
            service.Pause();
            server.StartUp();

            // assert deployed files and that iis is continued
            AssertDeployedFiles();
            service.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Running, service.Status, "IIS should be started");
        }

        /// <summary>
        /// Tests StartUp given a service that doesn't exist.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestStartUpServiceNotExist()
        {
            Option opt = new Option();
            opt.OptionName = IISServer.SERVICE_NAME;
            opt.Value = "NotExists";
            server.Options.Add(opt);
            server.StartUp();
        }

        /// <summary>
        /// Tests StartUp given a service that is already running.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestStartUpRunning()
        {
            ServiceHelper(ServiceControllerStatus.Running, service);
            server.StartUp();
        }

        /// <summary>
        /// Tests ShutDown. Assert that files are deleted and IIS service is stopped.
        /// </summary>
        [Test]
        public void TestShutDown()
        {
            // start the service first, deploy files, then shutdown
            server.StartUp();
            server.ShutDown();

            // assert deployed files are removed
            DirectoryInfo dir = new DirectoryInfo(DeployPath);
            Assert.AreEqual(0, dir.GetDirectories().Length, "Subdir removed");
            Assert.AreEqual(0, dir.GetFiles().Length, "All files removed");

            // assert iis is stopped
            service.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Stopped, service.Status, "IIS should be stopped");
        }

        /// <summary>
        /// Tests ShutDown given a service that doesn't exist.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestShutDownServiceNotExist()
        {
            Option opt = new Option();
            opt.OptionName = IISServer.SERVICE_NAME;
            opt.Value = "NotExists";
            server.Options.Add(opt);
            server.ShutDown();
        }

        /// <summary>
        /// Tests ShutDown given a service that is already stopped.
        /// Expecting ServerException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestShutDownStopped()
        {
            server.ShutDown();
        }

        /// <summary>
        /// Tests StartUp and ShutDown twice.
        /// Assert that files are deployed, IIS service is started, stopped, cleaned.
        /// If ShutDown modifies SourceFiles.BaseDirectory and fail to return it properly,
        /// the second deployment will fail.
        /// </summary>
        [Test]
        public void TestStartUpShutDownRepeatedly()
        {
            for (int i = 0; i < 2; i++)
            {
                // start and assert
                server.StartUp();

                // assert deployed files and that iis is started
                AssertDeployedFiles();
                service.Refresh();
                Assert.AreEqual(ServiceControllerStatus.Running, service.Status, "IIS should be started");

                // stop and assert
                server.ShutDown();

                // assert deployed files are removed
                DirectoryInfo dir = new DirectoryInfo(DeployPath);
                Assert.AreEqual(0, dir.GetDirectories().Length, "Subdir removed");
                Assert.AreEqual(0, dir.GetFiles().Length, "All files removed");

                // assert iis is stopped
                service.Refresh();
                Assert.AreEqual(ServiceControllerStatus.Stopped, service.Status, "IIS should be stopped");
            }
        }

        /// <summary>
        /// Tests InitData does nothing.
        /// Assert that files are not deployed and IIS service is still stopped.
        /// </summary>
        [Test]
        public void TestInitData()
        {
            server.InitData();

            // assert no deployed path
            DirectoryInfo dir = new DirectoryInfo(DeployPath);
            Assert.IsFalse(dir.Exists, "Deploy dir not created");

            // assert iis is still stopped
            service.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Stopped, service.Status, "IIS should still be stopped");
        }

        /// <summary>
        /// Tests CleanUpData does nothing.
        /// Assert that files are not deployed and IIS service is still stopped.
        /// </summary>
        [Test]
        public void TestCleanUpData()
        {
            server.CleanUpData();

            // assert no deployed path
            DirectoryInfo dir = new DirectoryInfo(DeployPath);
            Assert.IsFalse(dir.Exists, "Deploy dir not created");

            // assert iis is still stopped
            service.Refresh();
            Assert.AreEqual(ServiceControllerStatus.Stopped, service.Status, "IIS should still be stopped");
        }

        /// <summary>
        /// Asserts that files in test_files\deploysource are all deployed.
        /// </summary>
        private void AssertDeployedFiles()
        {
            string[] files = { "Hello.aspx", "Plain.txt", "sub\\Default.htm" };
            foreach (string file in files)
            {
                AssertFile(Path.Combine(SourcePath, file), Path.Combine(DeployPath, file));
            }
        }
    }
}
