/*
 * TestHelper.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.IO;
using System.ServiceProcess;
using System.Text;
using NUnit.Framework;

namespace TopCoder.Test.Framework.Tests
{
    /// <summary>
    /// Common values and functionality for the various unit tests.
    /// </summary>
    public class TestHelper
    {
        /// <summary>
        /// A sample NAnt project file to use.
        /// </summary>
        public const string SampleNAnt = @"..\..\test_files\ServerAppTest.build";

        /// <summary>
        /// Timeout for waiting a service to reach a certain state.
        /// </summary>
        public const int DefaultTimeOut = 5000;

        /// <summary>
        /// Reads the content of a fileName to a string.
        /// </summary>
        ///
        /// <param name="fileName">
        /// The file name whose content to read.
        /// </param>
        /// <param name="trim">
        /// Removes all white spaces character (including new line) at the beginning
        /// and end of every line in the file content.
        /// </param>
        ///
        /// <returns>
        /// The file content as a <see cref="string"/> or
        /// an empty string if the file doesn't exist or is empty.
        /// </returns>
        public static string ReadContent(string fileName, bool trim)
        {
            StringBuilder content = new StringBuilder();

            if (File.Exists(fileName))
            {
                // read the file content
                using (StreamReader reader = new StreamReader(fileName))
                {
                    string line;
                    while ( (line = reader.ReadLine()) != null )
                    {
                        // trim if asked
                        content.Append(trim ? line.Trim() : line);
                        if (!trim)
                        {
                            content.Append(Environment.NewLine);
                        }
                    }
                }
            }

            return content.ToString();
        }

        /// <summary>
        /// Asserts that two files have the same contents.
        /// </summary>
        ///
        /// <param name="file1">
        /// The 1st file path to compare.
        /// </param>
        /// <param name="file2">
        /// The 2nd file path to compare.
        /// </param>
        public static void AssertFile(string file1, string file2)
        {
            string c1 = ReadContent(file1, false);
            string c2 = ReadContent(file2, false);
            Assert.AreEqual(c1, c2, string.Format(
                "File {0} and {1} are different", file1, file2));
        }

        /// <summary>
        /// Deletes directory and all its contents recursively.
        /// </summary>
        ///
        /// <param name="path">
        /// The directory path to delete.
        /// </param>
        public static void DeleteDir(string path)
        {
            DirectoryInfo dir = new DirectoryInfo(path);
            if (dir.Exists)
            {
                dir.Delete(true);
            }
        }

        /// <summary>
        /// Starts or stops a service.
        /// </summary>
        ///
        /// <param name="expected">
        /// The expected status of the service. Only Running and Stopped is allowed.
        /// </param>
        /// <param name="service">
        /// The ServiceController to use.
        /// </param>
        public static void ServiceHelper(ServiceControllerStatus expected, ServiceController service)
        {
            service.Refresh();
            if (expected == ServiceControllerStatus.Stopped)
            {
                if (service.CanStop)
                {
                    service.Stop();
                }
            }
            else
            {
                if (service.Status == ServiceControllerStatus.Stopped)
                {
                    service.Start();
                }
            }
            service.WaitForStatus(expected, TimeSpan.FromMilliseconds(DefaultTimeOut));
        }
    }
}
